/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <DxcConfig.hpp>
#include <Connection/ScenarioLoaderConnector.hpp>

using namespace Dxc;
using namespace std;

ScenarioLoaderConnector::ScenarioLoaderConnector(DxcCallback *cb)
    : DxcConnectorImp(cb), sdsock_(-1), srsock_(-1)
{
    acceptConnections();
}

void ScenarioLoaderConnector::initConnections()
{
    connect("localhost", DxcConfig::getInstance()->getInt("SR_SL_PORT"), &srsock_);
}

bool ScenarioLoaderConnector::sendMessage(const DxcData * msg)
{
    bool rc = true;

    string msgType(typeid(*msg).name());
    if (msgType == typeid(ProfilingData).name())
    {
        if (srsock_ > 0)
            rc &= DxcConnectorImp::transmit(srsock_, msg);
    }
    else if (msgType == typeid(ErrorData).name() ||
             msgType == typeid(ScenarioStatusData).name())
    {
        if (srsock_ > 0)
            rc &= DxcConnectorImp::transmit(srsock_, msg);
        if (sdsock_ > 0)
            rc &= DxcConnectorImp::transmit(sdsock_, msg);
    }
    else
    {
        if (sdsock_ > 0)
            rc &= DxcConnectorImp::transmit(sdsock_, msg);
    }

    return rc;
}

void ScenarioLoaderConnector::acceptConnections()
{
    DxcConnectorImp::acceptConnection(DxcConfig::getInstance()->getInt("SL_SDS_PORT"), &sdsock_);
}
