/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef FAULT_INJECT_DATA_HPP
#define FAULT_INJECT_DATA_HPP

#include <set>
#include <map>
#include <vector>
#include <DxcDataTypes.hpp>
#include <DxcDefs.hpp>

namespace Dxc
{
/**
 * @brief Defines the data structure for fault injection information message.
 *
 * During an experiment, fault injection information is sent to the apropriate parties to
 * enable benchmarking of diagnosis algorithms. This information will be
 * inaccessible by diagnosers during competition scenarios. Unlike DiagnosisData,
 * FaultInjectData is used to communicate an *actual* faulty component and mode.
 */
class DXCAPI_EXPORT FaultInjectData : public DxcData
{
public:
    /**
     * @brief Constructor.
     *
     * @param timestamp - message time.
     * @param isInjection - was the fault injected (true), or removed (false)?
     * @param fault - the fault being injected/removed
     */
    FaultInjectData(Timestamp timestamp,
                    const Fault& fault,
                    bool isInjection);

    /**
     * @brief Was the fault injected (true) or removed (false)?
     */
    bool isInjection() const;

    /**
     * @brief The component that has entered or exited a faulty state.
     */
    Fault getFault() const;

    virtual FaultInjectData* clone() const;

    virtual std::ostream &put(std::ostream &) const;

private:
    FaultInjectData();

    bool isInjection_;
    Fault f_;
};
}

#endif

