/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <DataTypes/ScenarioData.hpp>

using namespace Dxc;
using namespace std;

ScenarioData::ScenarioData(Timestamp timestamp, const std::string& scenName,
                           const std::string& sysName, const std::string& sysVersion,
                           const std::string& daName, const std::string& daVersion,
                           const std::string& fwVersion,
                           long len, const std::string& notes)
    : DxcData(timestamp), scenName_(scenName), sysName_(sysName),
      sysVers_(sysVersion), daName_(daName), daVers_(daVersion),
      fwVers_(fwVersion), scenLen_(len), notes_(notes)
{

}

ScenarioData::ScenarioData(const std::string& scenName,
                           const std::string& sysName, const std::string& sysVersion,
                           const std::string& daName, const std::string& daVersion,
                           const std::string& fwVersion,
                           long len, const std::string& notes)
    : scenName_(scenName), sysName_(sysName),
      sysVers_(sysVersion), daName_(daName), daVers_(daVersion),
      fwVers_(fwVersion), scenLen_(len), notes_(notes)
{

}

ScenarioData* ScenarioData::clone() const
{
    return new ScenarioData(*this);
}

ostream &ScenarioData::put(ostream &out) const
{
    out << "scenarioInfo @" << this->getTimeStamp();

    if (this->getSource() != "")
        out << " <" << this->getSource() << ">";
    out << " scenarioName = '" << this->scenName_
        << "', system = '" << this->sysName_
        << "', systemVersion = '" << this->sysVers_
        << "', daName = '" << this->daName_
        << "', daVersion = '" << this->daVers_
        << "', frameworkVersion = '" << this->fwVers_
        << "', scenarioLength = " << this->scenLen_
        << ", notes = '" << this->notes_ << "';" << endl;
    out.flush();
    return out;
}
