/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <DataTypes/SensorData.hpp>

using namespace Dxc;
using namespace std;

SensorData::SensorData(Timestamp timestamp, const SensorValueMap& sensorMap)
    : DxcData(timestamp), sensorMap_(sensorMap)
{
}

ostream &SensorData::put(ostream & out) const
{
    out << "sensors @" << this->getTimeStamp();

    if (this->getSource() != "")
        out << " <" << this->getSource() << ">";

    out << " { ";

    bool fg = false;
    for (SensorValueMap::const_iterator i = sensorMap_.begin();
            i != sensorMap_.end();
            i++)
    {
        if (fg)
        {
            out << ", ";
        }
        else
        {
            fg = true;
        }
        out << i->first << " = " << i->second;
    }
    out << " };" << endl;

    return out;
}

const SensorData::SensorValueMap* SensorData::getSensors() const
{
    return &sensorMap_;
}

SensorData::SensorValueMap SensorData::getSensorValueMap() const
{
    return sensorMap_;
}

SensorData* SensorData::clone() const
{
    return new SensorData(*this);
}

SensorData::SensorData(const SensorData& sd)
{
    this->setTimeStamp(sd.getTimeStamp());

    SensorValueMap::const_iterator i = sd.getSensors()->begin();
    for (; i != sd.getSensors()->end(); i++)
        sensorMap_[i->first] = i->second->clone();
}

SensorData& SensorData::operator=(const SensorData& sd)
{
    if (this != &sd)
    {
        this->setTimeStamp(sd.getTimeStamp());

        SensorValueMap::iterator i = sensorMap_.begin();
        for (; i != sensorMap_.end(); i++)
            delete i->second;

        sensorMap_.clear();

        SensorValueMap::const_iterator j = sd.getSensors()->begin();
        for (; j != sd.getSensors()->end(); j++)
            sensorMap_[j->first] = j->second->clone();
    }

    return *this;
}

SensorData::~SensorData()
{
    SensorValueMap::iterator i = sensorMap_.begin();
    for (; i != sensorMap_.end(); i++)
        delete i->second;
}

SensorData::SensorData()
    : DxcData(0)
{
}
