/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.example;

import org.dxc.api.DxcCallback;
import org.dxc.api.connection.ConnectorFactory;
import org.dxc.api.connection.DxcConnector;
import org.dxc.api.datatypes.*;

/**
 * ExampleProbeDA demonstrates how a diagnosis algorithm should send probe
 * commands to obtain more sensor data for systems with probing (such as
 * synthetic systems). See ExampleDA for a more general example.
 */
public class ExampleJavaProbeDA {

    private static boolean run = true;
    private static final DxcCallback exampleCallback = new DxcCallback() {
        public void processData(DxcData d) {
            /*
             * d is incoming data. Find out what type it is, downcast it to that
             * DxcData type, and process accordingly.
             */
            if (d instanceof SensorData) {
                synchronized (this) {
                    // update observations and send diagnosis if needed
                    notifyAll();
                }
            } else if (d instanceof ScenarioStatusData) {
                // check for scenario end, exit if ended

                ScenarioStatusData stat = (ScenarioStatusData) d;
                if (stat.getStatus().equals(ScenarioStatusData.ENDED))
                    run = false;
            }
        }
    };

    public static void main(String[] args) {
        try {
            DxcConnector c = ConnectorFactory.getDAConnector(exampleCallback);
            c.sendMessage(new ScenarioStatusData(ScenarioStatusData.DA_READY));

            while (run) {
                Thread.sleep(3000);
            }

            // a simple strategy is to query all probe points at the end
            // of the scenario to resolve ambiguity:
            CommandSet cs = new CommandSet();
            cs.add(new Command("probe", Value.v("z1")));
            cs.add(new Command("probe", Value.v("z2")));
            cs.add(new Command("probe", Value.v("z3")));
            // et cetera...
            CommandData cd = new CommandData(cs);

            synchronized (exampleCallback) {
                c.sendMessage(cd);

                // wait for processData() to receive sensor data and send
                // diagnosis
                exampleCallback.wait();
            }
        } catch (Exception ex) {
            System.out.append(ex.toString() + " " + ex.getMessage());
        }

        System.exit(0);
    }

}
