package org.dxc.evaluator;

import java.text.DecimalFormat;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

import org.dxc.evaluator.EvalConfig;
import org.dxc.evaluator.metrics.*;

public class ScoreKeeper {
    public class DAComp implements Comparable<DAComp> {
        public MetricInfo compM;
        public DAResult dr;
        public boolean hib;

        DAComp(MetricInfo m, DAResult d) {
            this.compM = m;
            this.dr = d;
            this.hib = m.isHigherBetter();
        }

        public int compareTo(DAComp d1) {
            Double thisScore = this.dr.getMetricScore(compM);
            Double d1Score = d1.dr.getMetricScore(compM);

            if (thisScore.isNaN())
                return d1Score.isNaN() ? 0 : 1;
            else if (d1Score.isNaN())
                return -1;

            return !hib ? thisScore.compareTo(d1Score) : -thisScore
                    .compareTo(d1Score);
        }
    }

    private EvalConfig ec;

    public ScoreKeeper() throws Exception {
        ec = EvalConfig.getInstance();
    }

    public void scoreDAs(Collection<DAResult> dac) {
        TreeMap<DAResult, Double> scores = new TreeMap<DAResult, Double>();

        for (DAResult d : dac) {
            scores.put(d, 0.0);
        }

        for (MetricInfo m : ec.getMetrics()) {
            List<DAComp> dal = new LinkedList<DAComp>();
            for (DAResult d : dac) {
                dal.add(new DAComp(m, d));
            }

            Collections.sort(dal);

            if (ec.printStats())
                System.out.println("Metric ranking: " + m.getName());
            for (int i = 0; i < dal.size(); i++) {
                double totalPoints = ec.getRankPoints(i);
                List<DAResult> recipients = new LinkedList<DAResult>();
                recipients.add(dal.get(i).dr);

                while (i + 1 < dal.size()
                        && dal.get(i).compareTo(dal.get(i + 1)) == 0) {
                    i++;
                    recipients.add(dal.get(i).dr);
                    totalPoints += ec.getRankPoints(i);
                }

                double score = totalPoints / (double) recipients.size();
                score *= m.getWeight();

                for (DAResult d : recipients) {
                    double s = scores.get(d);
                    scores.put(d, s + score);

                    if (ec.printStats()) {
                        System.out.println((i + 1) + " - ("
                                + new DecimalFormat("0.0##").format(score)
                                + " pts) - " + d.getDAName());
                    }
                }
            }
        }

        System.out.println();
        System.out.println("Final Scores: ");
        for (DAResult d : scores.keySet()) {
            System.out.print(d.getDAName() + " - ");
            System.out
                    .println(new DecimalFormat("0.0##").format(scores.get(d)));
        }
    }
}