package org.dxc.evaluator.metrics;

import org.dxc.api.datatypes.DxcData;
import org.dxc.api.datatypes.FaultInjectData;
import org.dxc.api.datatypes.DiagnosisData;

/*
 * CPU Time metric.
 */
public class IsolationTime implements ScenarioMetricCalculator {
    private DiagnosisData lastDiagnosis;
    private long faultTime;
    private long isolationTime;
    private boolean isolated;

    IsolationTime() {
        lastDiagnosis = null;
        isolationTime = -1;
        faultTime = -1;
        isolated = false;
    }

    IsolationTime(DiagnosisData previousDiagnosis) {
        if (previousDiagnosis != null
                && previousDiagnosis.getCandidateSet().size() > 0)
            lastDiagnosis = previousDiagnosis;
        isolationTime = -1;
        faultTime = -1;
        isolated = false;
    }

    IsolationTime(DiagnosisData previousDiagnosis, long previousFaultTime) {
        if (previousDiagnosis != null
                && previousDiagnosis.getCandidateSet().size() > 0)
            lastDiagnosis = previousDiagnosis;
        isolationTime = -1;
        faultTime = previousFaultTime;
        isolated = false;
    }

    public void addData(DxcData d) {
        if (d instanceof FaultInjectData) {
            FaultInjectData f = (FaultInjectData) d;
            if (f.isInjection()) {
                if (faultTime < 0)
                    faultTime = f.getTimeStamp();
            }
        } else if (d instanceof DiagnosisData) {
            DiagnosisData dd = (DiagnosisData) d;
            if (lastDiagnosis != null) {
                if (!lastDiagnosis.getCandidateSet().equals(
                        dd.getCandidateSet())) {
                    lastDiagnosis = dd;
                    isolated = true;
                }
            } else if (dd.getCandidateSet().size() > 0) {
                lastDiagnosis = dd;
                isolated = true;
            }
        }
    }

    public boolean isolated() {
        return isolated;
    }

    public long getFaultTime() {
        return faultTime;
    }

    public void calculate() {
        if (faultTime >= 0 && lastDiagnosis != null
                && lastDiagnosis.getTimeStamp() >= faultTime) {
            isolationTime = lastDiagnosis.getTimeStamp() - faultTime;
        }
    }

    public double getScore() {
        return isolationTime < 0 ? Double.NaN : isolationTime;
    }

    public void print() {
        if (lastDiagnosis == null)
            return;
        System.out.print("Isolation");
        if (isolationTime < 0)
            System.out.println(" before fault injection.");
        else
            System.out.println(" @ time: " + isolationTime);
    }
}