package org.dxc.evaluator.metrics;

import java.util.LinkedList;
import java.util.ListIterator;

import org.dxc.api.datatypes.DiagnosisData;
import org.dxc.api.datatypes.DxcData;
import org.dxc.api.datatypes.FaultInjectData;
import org.dxc.api.datatypes.FaultSet;

/*
 * CPU Time metric.
 */
public class IsolationTimeList extends LinkedList<IsolationTime> implements
        ScenarioMetricCalculator {

    private static final long serialVersionUID = 1L;
    private FaultSet faultSet;
    private DiagnosisData lastDiagnosis;

    IsolationTimeList() {
        faultSet = new FaultSet();
        lastDiagnosis = null;
    }

    public void addData(DxcData d) {
        if (d instanceof FaultInjectData) {
            FaultInjectData fid = (FaultInjectData) d;
            if (fid.isInjection()) {
                faultSet.add(fid.getFault());
            } else {
                faultSet.remove(fid.getFault());
            }
            IsolationTime i, prev = null;
            if (!this.isEmpty())
                prev = this.getLast();
            ListIterator<IsolationTime> it = this.listIterator(this.size());

            while (it.hasPrevious()) {
                IsolationTime tmp = it.previous();
                if (tmp.isolated())
                    break;
                else if (tmp.getFaultTime() > -1)
                    prev = tmp;
            }

            if (prev == null || prev.isolated())
                i = new IsolationTime(lastDiagnosis);
            else
                i = new IsolationTime(lastDiagnosis, prev.getFaultTime());

            i.addData(fid);
            this.add(i);
        } else if (d instanceof DiagnosisData) {
            if (this.isEmpty())
                this.add(new IsolationTime(null));

            DiagnosisData dd = (DiagnosisData) d;
            if (lastDiagnosis != null) {
                if (!lastDiagnosis.getCandidateSet().equals(
                        dd.getCandidateSet()))
                    lastDiagnosis = dd;
            } else {
                lastDiagnosis = dd;
            }

            this.getLast().addData(dd);
        }
    }

    public void calculate() {
        for (IsolationTime i : this) {
            i.calculate();
        }
    }

    public double getScore() {
        return Double.NaN;
    }

    public void print() {
        for (IsolationTime i : this) {
            i.print();
        }
    }
}