package org.dxc.evaluator.metrics;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;

import org.dxc.api.DxcCallback;
import org.dxc.api.connection.ConnectorFactory;
import org.dxc.api.connection.DxcConnector;
import org.dxc.api.datatypes.DxcData;
import org.dxc.api.datatypes.ScenarioStatusData;

public class OracleInterface {
    private static OracleInterface instance;

    private DxcConnector oc;
    private Process p;
    private Worker w;
    private ForwardCallback internalCb = null;

    private class ForwardCallback extends DxcCallback {
        private DxcCallback c = null;

        public ForwardCallback(DxcCallback c) {
            this.c = c;
        }

        public void setCallback(DxcCallback c) {
            this.c = c;
        }

        public void processData(DxcData d) {
            if (c != null) {
                c.processData(d);
            }
        }
    }

    private OracleInterface(DxcCallback cb) {
        try {
            internalCb = new ForwardCallback(cb);
            oc = ConnectorFactory.getToOracleConnector(internalCb);
            synchronized (oc) {
                p = Runtime.getRuntime().exec(
                        System.getenv("DXC_HOME") + "/Bin/Oracle");
                oc.wait(3000);
                w = new Worker(p);
                w.start();
            }
        } catch (Exception ex) {
            System.out.println(ex);
        }
    }

    public static OracleInterface getInstance(DxcCallback cb) {
        if (null == instance) {
            createAnInstance(cb);
        } else {
            instance.internalCb.setCallback(cb);
        }
        return instance;
    }

    private synchronized static void createAnInstance(DxcCallback cb) {
        if (null == instance)
            instance = new OracleInterface(cb);
    }

    public DxcConnector getConnector() {
        return oc;
    }

    public static void shutdown() {
        if (instance == null)
            return;
        instance.oc.sendMessage(new ScenarioStatusData(
                ScenarioStatusData.DA_ENDED));

        try {
            instance.w.join(2000);
            if (instance.w.exit == null) {
                System.out.println("Oracle has not exited. Terminating.");
            }
        } catch (InterruptedException ex) {
            instance.w.interrupt();
            Thread.currentThread().interrupt();
        } finally {
            instance.p.destroy();
            instance.oc.shutdown();
            instance = null;
        }
    }

    public static void dumpOutput(OutputStream out) {
        String line;
        PrintStream p = new PrintStream(out);
        BufferedReader input = new BufferedReader(new InputStreamReader(
                instance.p.getInputStream()));

        try {
            while (input.ready() && (line = input.readLine()) != null) {
                p.println(line);
            }
        } catch (Exception ex) {
        }

        BufferedReader err = new BufferedReader(new InputStreamReader(
                instance.p.getErrorStream()));

        try {
            while (err.ready() && (line = err.readLine()) != null) {
                p.println(line);
            }
        } catch (Exception ex) {
        }
    }

    public static void clearOutput() {
        BufferedReader input = new BufferedReader(new InputStreamReader(
                instance.p.getInputStream()));

        try {
            while (input.ready() && input.readLine() != null) {
            }
        } catch (Exception ex) {
        }
    }

    private static class Worker extends Thread {
        private final Process process;
        private Integer exit = null;

        private Worker(Process process) {
            this.process = process;
        }

        public void run() {
            try {
                exit = process.waitFor();
            } catch (Exception ignore) {
                return;
            }
        }
    }
}
