/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.standaloneSDS;

import org.dxc.api.datatypes.Command;
import org.dxc.api.datatypes.CommandData;
import org.dxc.api.datatypes.DxcData;
import org.dxc.api.datatypes.SensorData;
import org.dxc.api.datatypes.StringValue;
import org.dxc.api.datatypes.StringValueMap;
import org.dxc.api.datatypes.Value;
import org.dxc.api.connection.DxcConnector;
import org.dxc.api.DxcCallback;

public class ProbeCallback extends DxcCallback {
    private DxcConnector connector;
    private SensorData lastSensorData;

    public ProbeCallback(DxcConnector connector) {
        this.connector = connector;
    }

    public void processData(DxcData data) {
        if (data instanceof SensorData) {
            lastSensorData = (SensorData) data;
        } else if (data instanceof CommandData) {
            CommandData cd = (CommandData) data;
            for (Command c : cd.getCommands()) {
                if (c.getCommandID().equals("probe")) {
                    try {
                        String sensor = ((StringValue) c.getValue()).get();
                        Value v = lastSensorData.getSensorValueMap()
                                .get(sensor);
                        StringValueMap svm = new StringValueMap();
                        svm.put(sensor, v);
                        SensorData sd = new SensorData(
                                DxcData.getCurrentTime(), svm);
                        connector.sendMessage(sd);
                    } catch (Exception ex) {
                    }
                }
            }
        }
    }
}
